# Copyright (C) 2012 Canonical Ltd.
# Copyright (C) 2012 Hewlett-Packard Development Company, L.P.
# Copyright (C) 2012 Yahoo! Inc.
#
# Author: Scott Moser <scott.moser@canonical.com>
# Author: Juerg Haefliger <juerg.haefliger@hp.com>
# Author: Joshua Harlow <harlowja@yahoo-inc.com>
# Author: Ben Howard <ben.howard@canonical.com>
#
# This file is part of cloud-init. See LICENSE file for license information.

# Author: Andrey Kirillov <akirillov@astralinux.ru>

"""
**Config keys**::

    users:
        - name: <username>

          # Level can be one of:
          #  <number> - explicitly specified level
          #  high     - current max_ilev
          #  max      - current max_ilev
          #  low      - equivalent to 0
          #  min      - equivalent to 0

          parsec_user_max_ilev: <parsec_max_ilev>
"""


from cloudinit import util
from cloudinit.distros import debian
from cloudinit.distros import astra_util
from cloudinit import helpers
from cloudinit import log as logging

LOG = logging.getLogger(__name__)


class Distro(debian.Distro):
    def __init__(self, name, cfg, paths):
        super(Distro, self).__init__(name, cfg, paths)
        # This will be used to restrict certain
        # calls from repeatly happening (when they
        # should only happen say once per instance...)
        self._runner = helpers.Runners(paths)
        self.osfamily = 'debian'
        self.default_locale = 'ru_RU.UTF-8'
        self.system_locale = None

    def create_user(self, name, **kwargs):
        """
        Creates a user and additionally sets parsec MIC
        settings for this user
        """
        result = False

        result = super().create_user(name, **kwargs)

        # Exit if the parsec MIC subsystem is missing or
        # not active.
        if not astra_util.mic_is_active():
            return result

        LOG.debug("parsec MIC is active")
 
        # sysmaxilev required only to replace string values
        sysmaxilev = astra_util.get_max_ilev()
        LOG.debug("sysmaxilev: {}".format(sysmaxilev))

        # cfgmaxilev
        if kwargs.get('parsec_user_max_ilev', False):
            # If the value is set in the configuration:
            # 1. Replacing string constants 'high' and 'max'
            # with a sysmaxilev value
            tmpstr = kwargs['parsec_user_max_ilev'].lower()
            if tmpstr.isnumeric():
                cfgmaxilev = tmpstr
            elif tmpstr in astra_util.PARSEC_ILEV_HIGH_NAMES:
                cfgmaxilev = sysmaxilev
            # 2. Replacing string constants 'low' and 'min'
            # with a zero value
            elif tmpstr in astra_util.PARSEC_ILEV_LOW_NAMES:
                cfgmaxilev = 0

            # TODO: if cfgmaxilev < 0 and cfgmaxilev > sysmaxilev: raise ConfigurationException()/ValueError('maxilev value out of range')

            # 3. Make the necessary settings using the pdp_* functions
            LOG.debug("Set max_ilev {} ({}) for '{}' user".format(cfgmaxilev, tmpstr, name))
            astra_util.pdp_user(name, cfgmaxilev)
            astra_util.pdp_file(astra_util.get_user_home(name), cfgmaxilev)

        return result

# vi: ts=4 expandtab
