# Author: Andrey Kirillov <akirillov@astralinux.ru>

"""
Helper functions for AstraLinux parsec MIC subsystem.
"""

import os

from cloudinit import log as logging
from cloudinit import subp, util

LOG = logging.getLogger(__name__)


PARSEC_ILEV_NAMES = ('low', 'min', 'high', 'max')
PARSEC_ILEV_HIGH_NAMES = ('high', 'max')
PARSEC_ILEV_LOW_NAMES = ('low', 'min')
MAX_ILEV_FILE_PATH='/sys/module/parsec/parameters/max_ilev'

def mic_is_active():
    """
    Returns True if parsec MIC subsystem is present and active, False otherwise
    """
    lang_c = os.environ.copy()
    lang_c['LANG'] = 'C'

    result = False

    checks = [
        os.path.exists(MAX_ILEV_FILE_PATH),
        subp.which('astra-mic-control') is not None
    ]
    if not all(checks):
        return result

    status = 'INACTIVE'
    cmd = ['astra-mic-control', 'status']
    cmd_rcs = (0, 1)
    try:
        # NOTE: 'astra-mic-control' command exit code 0 - if 'ACTIVE',
        # 1 - if 'INACTIVE'.
        out, err = subp.subp(cmd, capture=True, rcs=cmd_rcs, env=lang_c)
        status = util.decode_binary(out).strip('\n\r')
    except subp.ProcessExecutionError as e:
        err = util.decode_binary(e.stderr).lower()
        if (e.exit_code not in cmd_rcs):
            LOG.debug("*** Failed to execute '{}' command".format(' '.join(cmd)))
        else:
            util.logexc(LOG, "Failed to execute '{}' command".format(' '.join(cmd)))

    return 'ACTIVE' == status

def get_max_ilev():
    """
    Returns parsec MIC max_ilev value
    """
    sysmaxilev = 0
    if os.path.exists(MAX_ILEV_FILE_PATH):
        sysmaxilev = util.load_file(MAX_ILEV_FILE_PATH).strip('\r\n')
    return sysmaxilev

def pdp_user(name, max_ilev):
    """
    Set user max ilev

    name (str): user name
    max_ilev (int): max intergrity level
    """
    lang_c = os.environ.copy()
    lang_c['LANG'] = 'C'

    cmd = ['pdpl-user', '-i {}'.format(max_ilev), name]
    try:
        _, _ = subp.subp(cmd, capture=False, env=lang_c)
    except subp.ProcessExecutionError as e:
        err = util.decode_binary(e.stderr).lower()
        if (e.exit_code == 1):
            LOG.debug("Failed to execute '{}' command".format(' '.join(cmd)))
        else:
            util.logexc(LOG, "Failed to execute '{}' command".format(' '.join(cmd)))

def pdp_file(path, max_ilev):
    """
    Set FS object max ilev

    path (str) - file system object path

    Need to get the current value, and replace only the
    necessary component. But since the program is executed
    once at boot, you can reset the other components.
    pdpl-file "0:$maxilev:0:0" <path>
    """
    lang_c = os.environ.copy()
    lang_c['LANG'] = 'C'

    cmd = ['pdpl-file', '0:{}:0:0'.format(max_ilev), path]
    try:
        _, _ = subp.subp(cmd, capture=False, env=lang_c)
    except subp.ProcessExecutionError as e:
        err = util.decode_binary(e.stderr).lower()
        if (e.exit_code == 1):
            LOG.debug("Failed to execute '{}' command".format(' '.join(cmd)))
        else:
            util.logexc(LOG, "Failed to execute '{}' command".format(' '.join(cmd)))

def get_user_home(name):
    """
    Returns the user's home directory
    """
    lang_c = os.environ.copy()
    lang_c['LANG'] = 'C'

    result = ''

    database = 'passwd'
    cmd = ['getent', database, name]
    try:
        out, err = subp.subp(cmd, capture=True, env=lang_c)
    except subp.ProcessExecutionError as e:
        err = util.decode_binary(e.stderr).lower()
        if (e.exit_code == 2):
            LOG.debug("Name '{}' could not be found in '{}' database".format(name, database))
        else:
            util.logexc(LOG, "Failed to execute '{}' command".format(' '.join(cmd)))

    passwd_line = util.decode_binary(out).strip('\n\r')
    if passwd_line:
        result = [it.strip() for it in passwd_line.split(':')][5]
    LOG.debug("Home directory: '{}' for the '{}' user".format(result, name))

    return result

# vi: ts=4 expandtab
